// Generates the vertices, colors, and normals of a unit cube
// centered at origin, sides aligned with axes.
//
//    v5----- v6
//   /|      /|
//  v1------v2|              ^ y
//  | |     | |              |
//  | |v4---|-|v7            -->x
//  |/      |/              /
//  v0------v3              z


#include "Cube.h"
#include <iostream>
using std::cout;

Cube::Cube()
{
    Index = 0;

    vertices[0] = vec4( -0.5, -0.5,  0.5, 1.0 );   // v0
    vertices[1] = vec4( -0.5,  0.5,  0.5, 1.0 );   // v1
    vertices[2] = vec4(  0.5,  0.5,  0.5, 1.0 );   // v2
    vertices[3] = vec4(  0.5, -0.5,  0.5, 1.0 );   // v3
    vertices[4] = vec4( -0.5, -0.5, -0.5, 1.0 );   // v4
    vertices[5] = vec4( -0.5,  0.5, -0.5, 1.0 );   // v5
    vertices[6] = vec4(  0.5,  0.5, -0.5, 1.0 );   // v6
    vertices[7] = vec4(  0.5, -0.5, -0.5, 1.0 );    // v7

    vertex_colors[0]=   vec4( 0.0, 0.0, 0.0, 1.0 );  // black
    vertex_colors[1]= vec4( 1.0, 0.0, 0.0, 1.0 );  // red
    vertex_colors[2]= vec4( 1.0, 1.0, 0.0, 1.0 ),  // yellow
    vertex_colors[3]= vec4( 0.0, 1.0, 0.0, 1.0 );  // green
    vertex_colors[4]= vec4( 0.0, 0.0, 1.0, 1.0 );  // blue
    vertex_colors[5]= vec4( 1.0, 0.0, 1.0, 1.0 );  // magenta
    vertex_colors[6]= vec4( 1.0, 1.0, 1.0, 1.0 );  // white
    vertex_colors[7]= vec4( 0.0, 1.0, 1.0, 1.0 );   // cyan


    colorcube();

    /*
    cout << "points:\n";
    for (int i = 0; i < 36; i++)
    {
        cout<< i << ":  " <<  points[i] << "\n";
    }
    cout << "colors:\n";
    for (int i = 0; i < 36; i++)
    {
        cout<< i << ":  " <<  colors[i] << "\n";
    }
    */
}

Cube::Cube(const Cube& other)
{
    for (int i=0; i < 36; i++)
    {
        points[i]=other.points[i];
        colors[i]=other.colors[i];
    }
    numVertices = other.numVertices;
}

Cube&
Cube::operator=(const Cube& rhs)
{
    if (this == &rhs) return *this; // handle self assignment
    //assignment operator
    return *this;
       for (int i=0; i < 36; i++)
    {
        points[i]=rhs.points[i];
        colors[i]=rhs.colors[i];
    }
    numVertices = rhs.numVertices;
}

Cube::~Cube()
{}


int Index = 0;

void
Cube::quad( int a, int b, int c, int d , int col)
{
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[a];
    Index++;
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[b];
    Index++;
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[c];
    Index++;
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[a];
    Index++;
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[c];
    Index++;
    colors[Index] = vertex_colors[col];
    points[Index] = vertices[d];
    Index++;
}

// generate 12 triangles: 36 vertices and 36 colors
//    v5----- v6
//   /|      /|
//  v1------v2|              ^ y
//  | |     | |              |
//  | |v4---|-|v7            -->x
//  |/      |/              /
//  v0------v3              z
// Always use the Right Hand Rule to generate vertex sequence. We want outward facing normals.
void
Cube::colorcube()
{
    // vertices   col norm
    quad( 1, 0, 3, 2 , 2);   // front (+z) face
    quad( 2, 3, 7, 6 , 1);   // +x
    quad( 3, 0, 4, 7 , 3);   // -y
    quad( 6, 5, 1, 2 , 4);   // +y
    quad( 4, 5, 6, 7 , 5);   // -z
    quad( 5, 4, 0, 1 , 6);   // -x

    numVertices = Index;
}


void
Cube::createVAO(GLint _vao, GLint program )
{
    vao = _vao;
    glBindVertexArray( vao );
    GLuint buffer;
    glGenBuffers( 1, &buffer );
    glBindBuffer( GL_ARRAY_BUFFER, buffer );

    int csize = sizeof(points);
   // cout << "Cube csize " << csize << "\n";
    glBufferData( GL_ARRAY_BUFFER, 2*csize,
                  NULL, GL_STATIC_DRAW );
    glBufferSubData( GL_ARRAY_BUFFER, 0, csize, points );
    glBufferSubData( GL_ARRAY_BUFFER, csize, csize, colors );

    // set up shader variables
    GLint vPosition = glGetAttribLocation( program, "vPosition" );
    glEnableVertexAttribArray( vPosition );
    glVertexAttribPointer( vPosition, 4, GL_FLOAT, GL_FALSE, 0,
                           BUFFER_OFFSET(0) );

    GLint vColor = glGetAttribLocation( program, "vColor" );
    glEnableVertexAttribArray( vColor );
    glVertexAttribPointer( vColor, 4, GL_FLOAT, GL_FALSE, 0,
                           BUFFER_OFFSET(csize) );
    glBindBuffer( GL_ARRAY_BUFFER, 0 );
    glBindVertexArray( 0 );
}

void Cube::draw() {
    glBindVertexArray( vao );
    glDrawArrays( GL_TRIANGLES, 0, numVertices );
    glBindVertexArray( 0 );
}





