//
// Displays a disk with different colored triangle regions. The user can rotate around the
//    x and y axis by pressing the "x" and "y" keys, or hit "r" to reset.

#include "Angel.h"
#include <iostream>
using std::cout;

#include "Cube.h"

#include "MatrixStack.h"

// Viewing transformation parameters
GLfloat theta = 0.0;
GLfloat phi = 0.0;
GLfloat alpha = 0.0;

// Robot Arm Angles
GLfloat aphi = 0.0;  // lower arm around normal
GLfloat theta1 = 0.0; // lower arm angle off of normal
GLfloat theta2 = 0.0; // middle arm relative to lower arm
GLfloat theta3 = 0.0; // upper arm relative to middle arm
GLfloat armLength = 2;

Cube myCube;  // generates the vertices, colors, normals of a cube.

// Cmamera projection transformation parameters
GLfloat  fovy = 45.0;  // Field-of-view in Y direction angle (in degrees)
GLfloat  aspect;       // Viewport aspect ratio
GLfloat  zNear = 0.5, zFar = 100.0;

GLuint  projection; // projection matrix uniform shader variable location

// Camera location and orientation parameters

vec4 eye = vec4( 0.0 ,5.0, 14.0 ,1.0); // camera location
vec4  att( 0.0, 3.0, 0.0, 0.0 );
vec4  vup( 0.0, 1.0, 0.0, 0.0);

GLuint  model_view;  // model-view matrix uniform shader variable location

MatrixStack mvMatrixStack;  // stores the movel view matrix stack

//----------------------------------------------------------------------------
GLuint program;
// OpenGL initialization
void
init()
{

    program = InitShader( "vertex.glsl", "fragment.glsl" );
    glUseProgram( program );
    // Uniform variables: color and viewing parameters
    model_view = glGetUniformLocation( program, "model_view" );
    projection = glGetUniformLocation( program, "projection" );

    // Create multiple vertex array objects
    GLuint vao[1];
    glGenVertexArrays( 1, vao );

    //********  Create the VAOs ************//

    myCube.createVAO(vao[0],program);

    glLineWidth(4);
    glEnable( GL_DEPTH_TEST );
    glClearColor( 0.5, 0.5, 0.5, 1.0 );
}

//----------------------------  draw base

void drawBase(mat4 &mv)
{
    mvMatrixStack.pushMatrix(mv);
    mv = mv*Translate(0,-.1,0);
    mv = mv*Scale(10,.2,10);
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv );
    myCube.draw();
    mv = mvMatrixStack.popMatrix();
}



//----------------------------------------------------------------------------

void
display( void )
{
    // clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // calculate camera projection parameters:
    mat4  p  = Perspective( fovy, aspect, zNear, zFar );
    // set the value of the projection matrix uniform variable for current shader program:
    glUniformMatrix4fv( projection, 1, GL_TRUE, p );
    // initialize the modelview matrix stack
    mvMatrixStack.loadIdentity();
    // calculate view transformations:  camera location and orientation -
    mat4 mv = LookAt( eye, att, vup );  //
    // calculate model transformations -
    mv = mv*RotateZ(alpha);   // rotate about z axis
    mv = mv*RotateY(theta);   // rotate about y axis
    mv = mv*RotateX(phi);     // rotate about x axis

    // save the current modelview matrix on the stack
    mvMatrixStack.pushMatrix(mv);
    // set the value of the modelview matrix uniform variable for current shader program:
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv );
    // draw the cube - the parameter is the cube color

    drawBase(mv);

    // unbind the current vertex array buffer
    glBindVertexArray( 0 );
    glutSwapBuffers();
}

//----------------------------------------------------------------------------


void
keyboard( unsigned char key, int x, int y )
{

    float inc = 5.0;
    switch( key )
    {
    case 033: // Escape Key
    case 'q':
    case 'Q':
        exit( EXIT_SUCCESS );
        break;

    case 'y':   // rotate around y
        theta += inc;
        break;
    case 'x':     // rotate around x  (this is always applied second)
        phi += inc;
        break;
    case 'z':     // rotate around z  (this is always applied second)
        alpha += inc;
        break;
    case 'a':     //
        aphi += inc;
        break;
    case 's':     //
        if (theta1 < 70) theta1 += inc;
        break;
    case 'd':     //
        theta2 += inc;
        break;
    case 'f':     //
        theta3 += inc;
        break;
    case 'A':     //
        aphi -= inc;
        break;
    case 'S':     //
        if (theta1 > -70) theta1 -= inc;
        break;
    case 'D':     //
        theta2 -= inc;
        break;
    case 'F':     //
        theta3 -= inc;
        break;
    case 'r':     // reset
        phi = 0;
        theta = 0;
        alpha = 0;
        aphi = 0;
        theta1 = 0;
        theta2 = 0;
        theta3 = 0;
        break;
    }

    glutPostRedisplay();
}


//----------------------------------------------------------------------------

void
reshape( int width, int height )
{
    glViewport( 0, 0, width, height );
    aspect = GLfloat(width)/height;
}

//----------------------------------------------------------------------------

int
main( int argc, char **argv )
{
    glutInit( &argc, argv );
    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    glutInitWindowSize( 512, 512 );
    glutInitContextVersion( 3, 3 );
    glutInitContextProfile( GLUT_CORE_PROFILE );
    glutCreateWindow( "Robot Arm" );

    glewInit();

    init();

    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    glutReshapeFunc( reshape );

    glutMainLoop();
    return 0;
}
